!
!-----------------------------------------------------------------------
!     Subroutine KT17_TAILDISK
!-----------------------------------------------------------------------
!
subroutine kt17_taildisk(xmsm,ymsm,zmsm,bx,by,bz)

! calculates msm components of the field from a t01-like 'long-module' equatorial current disk
! with a 'hole' in the center and a smooth inner edge
! (see tsyganenko, jgra, v.107,no.a8, doi 10.1029/2001ja000219, 2002, fig.1, right panel).
!
!------------input parameters:
!
! d0       - basic (minimal) half-thickness
! deltadx  - sunward expansion factor for the current sheet thickness
! deltady  - flankward expansion factor for the current sheet thickness
! x,y,z    - msm coordinates
!
!------------output parameters:
! bx,by,bz - field components in msm system, in nanotesla.

!! Tsyganenko 2002
!  f=[-71.09346626d0,-1014.30860d0,-1272.939359d0,-3224.935936d0,-44546.86232d0]
!  b=[10.90101242d0,12.68393898d0,13.51791954d0,14.86775017d0,15.12306404d0]
!  c=[.7954069972d0,.6716601849d0,1.174866319d0,2.565249920d0,10.01986790d0]

!! Tsyganenko t93ame5
!  f=[38034.54566d0,3434212.993d0,354620.7757d0,-3805876.878d0,-514473.9612d0]
!  b=[8.152293205d0,14.27689171d0,13.78960037d0,14.23483658d0,0.9298378229d-1]
!  c=[10.68511581d0,3.446457863d0,3.520494461d0,3.460958481d0,29.69889832d0]

  implicit none
  
  include 'kt17_common.f90'

  integer*4   nr3,i
  real*8      xmsm,ymsm,zmsm,bx,by,bz
  real*8      f(n_taildisk),b(n_taildisk),c(n_taildisk)
  real*8      xshift,sc,x,y,z,d0_sc,deltadx_sc,deltady_sc
  real*8      rho,drhodx,drhody
  real*8      dex,d,dddy,dddx
  real*8      dzeta,ddzetadx,ddzetady,ddzetadz
  real*8      bi,ci,s1,s2,ds1drho
  real*8      ds2drho,ds1ddz,ds2ddz
  real*8      ds1dx,ds1dy,ds1dz,ds2dx,ds2dy,ds2dz
  real*8      s1ts2,s1ps2,s1ps2sq,fac1,as,dasds1,dasds2
  real*8      dasdx,dasdy,dasdz

  nr3=n_taildisk/3
  f(1:nr3)=r_taildisk(1:nr3)
  b(1:nr3)=r_taildisk(nr3+1:2*nr3)
  c(1:nr3)=r_taildisk(2*nr3+1:3*nr3)

  xshift=0.3d0             ! shift the center of the disk to the dayside by xshift
  sc=7.0d0                 ! renormalize length scales
  
  x=(xmsm-xshift)*sc
  y=ymsm*sc
  z=zmsm*sc
  d0_sc=d0*sc
  deltadx_sc=deltadx*sc
  deltady_sc=deltady*sc
  
  rho=sqrt(x**2+y**2)
  drhodx=x/rho
  drhody=y/rho
  
  dex=exp(x/7.0d0)
  d=d0_sc+deltady_sc*(y/20.0d0)**2+deltadx_sc*dex   ! the last term (introduced 10/11/2000) makes the sheet
  dddy=deltady_sc*y*0.005d0                         ! thicken sunward, to avoid problems in the subsolar region
  dddx=deltadx_sc/7.0d0*dex
  
  dzeta=sqrt(z**2+d**2)                              ! this is to spread out the sheet in z direction over finite thickness 2d
  ddzetadx=d*dddx/dzeta
  ddzetady=d*dddy/dzeta
  ddzetadz=z/dzeta
  
  bx=0.0d0
  by=0.0d0
  bz=0.0d0
  
  do i=1,5
    bi=b(i)
    ci=c(i)
    
    s1=sqrt((rho+bi)**2+(dzeta+ci)**2)
    s2=sqrt((rho-bi)**2+(dzeta+ci)**2)
    
    ds1drho=(rho+bi)/s1
    ds2drho=(rho-bi)/s2
    ds1ddz=(dzeta+ci)/s1
    ds2ddz=(dzeta+ci)/s2
    
    ds1dx=ds1drho*drhodx+ds1ddz*ddzetadx
    ds1dy=ds1drho*drhody+ds1ddz*ddzetady
    ds1dz=ds1ddz*ddzetadz
    
    ds2dx=ds2drho*drhodx+ds2ddz*ddzetadx
    ds2dy=ds2drho*drhody+ds2ddz*ddzetady
    ds2dz=ds2ddz*ddzetadz
    
    s1ts2=s1*s2
    s1ps2=s1+s2
    s1ps2sq=s1ps2**2
    
    fac1=sqrt(s1ps2sq-(2.0d0*bi)**2)
    as=fac1/(s1ts2*s1ps2sq)
    dasds1=(1.0d0/(fac1*s2)-as/s1ps2*(s2*s2+s1*(3.0d0*s1+4.0d0*s2)))/(s1*s1ps2)
    dasds2=(1.0d0/(fac1*s1)-as/s1ps2*(s1*s1+s2*(3.0d0*s2+4.0d0*s1)))/(s2*s1ps2)
    
    dasdx=dasds1*ds1dx+dasds2*ds2dx
    dasdy=dasds1*ds1dy+dasds2*ds2dy
    dasdz=dasds1*ds1dz+dasds2*ds2dz
    
    bx=bx-f(i)*x*dasdz
    by=by-f(i)*y*dasdz
    bz=bz+f(i)*(2.0d0*as+x*dasdx+y*dasdy)
  enddo

  return
end
